local AddonName, GGL = ...

local AceGUI = LibStub("AceGUI-3.0")
local Templates = GGL.Templates
local L = GGL.Locale

-- lua


-- wow

local Multibox = {}
GGL.GUI.Multibox = Multibox

local GetMoneyText = GGL.GetMoneyText
local CreateLine = GGL.GUI.CreateLine

-- ##################
-- Window Creation
-- ##################
local WIDTH_V, HEIGHT_V = 300, 350
local WIDTH_M, HEIGHT_M = 0, 500
local CHAR_MAX_ENTRYS_HEIGHT = 10

local function SetMoneyText(self, money)
    self:SetText(GetMoneyText(money))
end

local FRAME_FUNCTIONS = {
    ["AddPlayer"] = function(self, player)
        if not player or self.RowList[player] then return end

        -- row creation
        local row = AceGUI:Create("GoldGoblinLootGUIWidget-MBChar")
        row:SetFullWidth(true)
        row:SetPlayer(player)
        if player == Templates.WoWPlayer:GetPlayer() then
            if self.leader and self.leader ~= row then
                self.leader:IsLeader(false)
            end
            row:IsLeader(true)
            self.leader = row
        else
            row:IsLeader(false)
        end
        self.charContainer:AddChild(row)
        row:SetToolTipPoint("BOTTOM_X", self.frame)

        self.RowList[player] = row

        -- set new hight
        local numChildren = #self.charContainer.children
        if numChildren <= CHAR_MAX_ENTRYS_HEIGHT then
            self.charContainer:SetHeight(row.frame:GetHeight() * numChildren)
        end
    end,

    ["GetPlayerRow"] = function(self, player)
        return self.RowList[player]
    end,

    ["SetUpdateInterval"] = function(self, interval)
        self.charContainer:SetRefreshInterval(interval)
        self.itemContainer1:SetRefreshInterval(interval)
        self.itemContainer2:SetRefreshInterval(interval)
    end,

    ["ClearPlayerList"] = function(self)
        self.charContainer:ReleaseChildren()
        wipe(self.RowList)
        self.charContainer:SetHeight(10)
    end,

    ["ClearItemLists"] = function(self)
        self.itemContainer1:ReleaseChildren()
        self.itemContainer2:ReleaseChildren()

        wipe(self.itemList)
        wipe(self.itemContainer1.SourceNames)
        wipe(self.itemList2)
        wipe(self.itemContainer2.SourceNames)
    end,

    ["Clear"] = function(self)
        -- clear lists
        self:ClearItemLists()
        self:SetUpdateInterval()
        self:ClearPlayerList()

        -- set gold
        self:SetValueGold(0)
        self:SetRawGold(0)
        self:SetTotalGold(0)
    end,

    ["SetFilter"] = function(self, mainSet, specialItemFilter)
        self.sets = self.sets or {}

        self.mainSet = mainSet or "Base"
        self.specialItemFilter = specialItemFilter
    end,

    ["AddItem"] = function(self, itemID, amount, moneyValue, sourcePlayer)
        if not moneyValue or moneyValue <= 0 then return end
        if not self.itemList[itemID] then
            local item = AceGUI:Create("GoldGoblinLootGUIWidget-CountItemLine")
            item:SetItem(itemID)
            item:SetMoneyValue(moneyValue)
            item:SetFullWidth(true)
            item:EnableBG(true)
            self.itemContainer1:AddChild(item)
            item:SetToolTipPoint("BOTTOM_X", self.frame)

            self.itemList[itemID] = item
        end

        self.itemList[itemID]:SetAmount(amount)

        if sourcePlayer then
            if not self.itemContainer1.SourceNames[itemID] then self.itemContainer1.SourceNames[itemID] = {} end
            self.itemContainer1.SourceNames[itemID][sourcePlayer.syncID] = (self.itemContainer1.SourceNames[itemID][sourcePlayer.syncID] or 0) + amount
        end
    end,

    ["AddItemSpecial"] = function(self, itemID, amount, moneyValue, sourcePlayer)
        if not moneyValue or moneyValue <= 0 then return end
        if not self.itemList2[itemID] then
            local item = AceGUI:Create("GoldGoblinLootGUIWidget-CountItemLine")
            item:SetItem(itemID)
            item:SetMoneyValue(moneyValue)
            item:SetFullWidth(true)
            item:EnableBG(true)
            self.itemContainer2:AddChild(item)
            item:SetToolTipPoint("BOTTOM_X", self.frame)

            self.itemList2[itemID] = item
        end

        self.itemList2[itemID]:SetAmount(amount)

        if sourcePlayer then
            if not self.itemContainer2.SourceNames[itemID] then self.itemContainer2.SourceNames[itemID] = {} end
            self.itemContainer2.SourceNames[itemID][sourcePlayer.syncID] = (self.itemContainer2.SourceNames[itemID][sourcePlayer.syncID] or 0) + amount
        end
    end,

    ["SetPlayerGold"] = function(self, player, copper, add)
        if not player or not self.RowList[player] then return end
        self.RowList[player]:SetMoney(copper, add)
    end,

    ["SetValueGold"] = function(self, copper, add)
        self.valueGoldCount = add and (self.valueGoldCount or 0) + copper or copper
        self.contentContainer.valueGold:SetMoneyText(self.valueGoldCount)
        self:SetTotalGold((self.rawGoldCount or 0) + (self.valueGoldCount or 0))
    end,

    ["SetRawGold"] = function(self, copper, add)
        self.rawGoldCount = add and (self.rawGoldCount or 0) + copper or copper
        self.contentContainer.rawGold:SetMoneyText(self.rawGoldCount)
        self:SetTotalGold((self.rawGoldCount or 0) + (self.valueGoldCount or 0))
    end,

    ["SetTotalGold"] = function(self, copper, add)
        self.sumGoldCount = add and (self.sumGoldCount or 0) + copper or copper
        self.contentContainer.sumGold:SetMoneyText(self.sumGoldCount)
    end,
}

function Multibox:CreateFrame()
    if not self.frame then
        local window = AceGUI:Create("Window")
        GGL.GUI_FixCloseButton(window.closebutton)
        window:SetTitle("GoldGoblinLoot - Multibox")

        local frame = window.frame

        -- some frame settings
        if frame.SetResizeBounds then
			frame:SetResizeBounds(WIDTH_V,HEIGHT_V,WIDTH_V+WIDTH_M,HEIGHT_V+HEIGHT_M)
		else
            frame:SetMinResize(WIDTH_V,HEIGHT_V)
            frame:SetMaxResize(WIDTH_V+WIDTH_M,HEIGHT_V+HEIGHT_M)
        end
        frame:SetClampedToScreen(true)
        frame:SetToplevel(false)
        frame:SetFrameStrata("MEDIUM")
        --window settings
        window:SetWidth(WIDTH_V)
        window:SetHeight(HEIGHT_V)

        --charContainerBG
        local charContainerBG = GGL.GUI:CreateBorderedFrame(frame)
        -- charContainer
        local charContainer = AceGUI:Create("GoldGoblinLootGUIContainer-SortedScrollFrame")
        charContainer:SetFullWidth(true)
        charContainer:SetHeight(70)
        window.charContainer = charContainer
        window:AddChild(charContainer)
        -- charContainer / bg
        charContainerBG:SetPoint("TOPLEFT", charContainer.scrollframe, -5, 5)
        charContainerBG:SetPoint("BOTTOMRIGHT", charContainer.scrollframe, 5, -5)

        -- itemContainer1
        local itemContainer1BG = GGL.GUI:CreateBorderedFrame(frame)
        local itemContainer1 = AceGUI:Create("GoldGoblinLootGUIContainer-SortedScrollFrame")
        itemContainer1.frame:SetParent(frame)
        itemContainer1:SetPoint("TOPLEFT", charContainer.scrollframe, "BOTTOMLEFT", 0, -10)
        itemContainer1:SetPoint("BOTTOM", window.content, 0, 5)
        itemContainer1:SetPoint("RIGHT", window.content, "CENTER", -5, 0)
        itemContainer1.SourceNames = {}
        window.itemContainer1 = itemContainer1
        -- charContainer / bg
        itemContainer1BG:SetPoint("TOPLEFT", itemContainer1.scrollframe, -5, 5)
        itemContainer1BG:SetPoint("BOTTOMRIGHT", itemContainer1.scrollframe, 5, -5)

        -- ## CONTENT
        local contentContainerF = GGL.GUI:CreateBorderedFrame(frame)
        local contentContainer = CreateFrame("Frame", nil, contentContainerF)
        contentContainer:SetPoint("BOTTOMLEFT", itemContainer1.frame, "BOTTOMRIGHT", 10, 0)
        contentContainer:SetPoint("RIGHT", window.content, -0, 0)
        contentContainer:SetPoint("BOTTOM", window.content, 0, 5)
        contentContainer:SetHeight(93)
        window.contentContainer = contentContainer
        contentContainerF:SetPoint("TOPLEFT", contentContainer, -5, 5)
        contentContainerF:SetPoint("BOTTOMRIGHT", contentContainer, 5, -5)

        -- content:Timer
        local timer = AceGUI:Create("GoldGoblinLootGUIWidget-Timer")
        timer.frame:SetParent(contentContainer)
        timer:SetPoint("TOP", contentContainer, "TOP", 0, 0)
        timer:SetDuration(0)
        contentContainer.timer = timer

        -- content: RawGold
        local rawGold = contentContainer:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmall")
        rawGold:SetPoint("TOPLEFT",contentContainer,0,-(timer.frame:GetHeight()+1))
        rawGold:SetPoint("RIGHT", contentContainer, 0, 0)
        rawGold:SetHeight(23)
        rawGold:SetJustifyH("CENTER")
        rawGold:SetJustifyV("BOTTOM")
        rawGold.SetMoneyText = SetMoneyText
        contentContainer.rawGold = rawGold

        local rawGoldTitle = contentContainer:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmall")
        rawGoldTitle:SetAllPoints(rawGold)
        rawGoldTitle:SetJustifyH("LEFT")
        rawGoldTitle:SetJustifyV("TOP")
        rawGoldTitle:SetText(L["Raw gold"]..":")
        contentContainer.rawGoldTitle = rawGoldTitle

        -- content: ValueGold
        local valueGold = contentContainer:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmall")
        valueGold:SetPoint("TOPLEFT", rawGold, "BOTTOMLEFT", 0, -1)
        valueGold:SetPoint("TOPRIGHT", rawGold, "BOTTOMRIGHT", 0, -1)
        valueGold:SetHeight(23)
        valueGold:SetJustifyH("CENTER")
        valueGold:SetJustifyV("BOTTOM")
        valueGold.SetMoneyText = SetMoneyText
        contentContainer.valueGold = valueGold

        local valueGoldTitle = contentContainer:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmall")
        valueGoldTitle:SetAllPoints(valueGold)
        valueGoldTitle:SetJustifyH("LEFT")
        valueGoldTitle:SetJustifyV("TOP")
        valueGoldTitle:SetText(L["Item value gold:"])
        contentContainer.valueGoldTitle = valueGoldTitle

        -- content: SumGold
        local sumGold = contentContainer:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmall")
        sumGold:SetPoint("TOPLEFT", valueGold, "BOTTOMLEFT", 0, -1)
        sumGold:SetPoint("TOPRIGHT", valueGold, "BOTTOMRIGHT", 0, -1)
        sumGold:SetHeight(23)
        sumGold:SetJustifyH("CENTER")
        sumGold:SetJustifyV("BOTTOM")
        sumGold.SetMoneyText = SetMoneyText
        contentContainer.sumGold = sumGold

        local sumGoldTitle = contentContainer:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmall")
        sumGoldTitle:SetAllPoints(sumGold)
        sumGoldTitle:SetJustifyH("LEFT")
        sumGoldTitle:SetJustifyV("TOP")
        sumGoldTitle:SetText(L["Total gold:"])
        contentContainer.sumGoldTitle = sumGoldTitle

        -- itemContainer2
        local itemContainer2BG = GGL.GUI:CreateBorderedFrame(frame)
        local itemContainer2 = AceGUI:Create("GoldGoblinLootGUIContainer-SortedScrollFrame")
        itemContainer2.frame:SetParent(frame)
        itemContainer2:SetPoint("TOPLEFT", itemContainer1.frame, "TOPRIGHT", 10, 0)
        itemContainer2:SetPoint("BOTTOM", contentContainer, "TOP", 10, 10)
        itemContainer2:SetPoint("TOPRIGHT", charContainer.frame, "BOTTOMRIGHT")
        itemContainer2.SourceNames = {}
        --itemContainer2:SetPoint("RIGHT", window.content, -5, 0)
        window.itemContainer2 = itemContainer2
        -- charContainer / bg
        itemContainer2BG:SetPoint("TOPLEFT", itemContainer2.scrollframe, -5, 5)
        itemContainer2BG:SetPoint("BOTTOMRIGHT", itemContainer2.scrollframe, 5, -5)


        -- lines
        local lines = {
            -- Timer bottom
            CreateLine(contentContainer, true, {"TOPLEFT",nil,0,-(timer.frame:GetHeight())}, {"TOPRIGHT",nil,0,-(timer.frame:GetHeight())}, nil, nil),
            -- Raw gold bottom
            CreateLine(contentContainer, true, {"TOPLEFT",rawGold,"BOTTOMLEFT"}, {"TOPRIGHT",rawGold,"BOTTOMRIGHT"}, nil, nil),
            -- Value gold bottom
            CreateLine(contentContainer, true, {"TOPLEFT",valueGold,"BOTTOMLEFT"}, {"TOPRIGHT",valueGold,"BOTTOMRIGHT"}, nil, nil),
            -- Sum gold bottom
            --CreateLine(contentContainer, true, {"TOPLEFT",sumGold,"BOTTOMLEFT"}, {"TOPRIGHT",sumGold,"BOTTOMRIGHT"}, nil, nil),
        }

        -- register functions
        for k,v in pairs(FRAME_FUNCTIONS) do
            window[k] = v
        end
        FRAME_FUNCTIONS = nil

        window.RowList = {}
        window.itemList = {}
        window.itemList2 = {}

        self.frame = window

        GGL.GUI:AddFrameControl("Multibox", frame, true, true)

        window:Hide()
    end
    return self.frame
end

function Multibox:Show()
    if not self.frame then self:CreateFrame() end
    self.frame:Show()
end

function Multibox:AddIntoMiniMapButton()
    GGL:RegisterMMFunction(self, self.Show)
end

function Multibox:RemoveFromMiniMapButton()
    GGL:UnregisterMMFunction(self.Show)
end