local AddonName, GGL = ...

local AceGUI = LibStub("AceGUI-3.0")
local Templates = GGL.Templates
local L = GGL.Locale

local SpecialEvent = {}
GGL.GUI.SpecialEvent = SpecialEvent

-- lua

-- wow

-- GGL
local GetPlayerTemplate = Templates.Sync.GetPlayerFromSync

-- ##################
-- shared
-- ##################


-- ##################
-- Base
-- ##################


-- ##################
-- Opening
-- ##################
local OpeningFrame
local OPENING_SOURCE_LIST_REFRESH_INTERVAL = 5
local OPENING_FRAME_DEFAULTS = {
    MIN_WIDTH = 500,
    MAX_WIDTH = 500,
    MIN_HEIGHT = 500,
    MAX_HEIGHT = 800,
    BOX_GAP = 10
}
local LastLeftSelectedItemRow = nil

local function ItemContainerLeft_BeforeListSort(self, event)
    -- ??
end

local function OpeningSortFunction(a, b)
    if a.goldValue ~= b.goldValue then
        return (a.goldValue or 0) > (b.goldValue or 0)
    end

    return (a.item or "") < (b.item or "")
end

local function LeftItemRow_SelectStateChange(self, event, state)
    -- lock state when clicked again
    if not state and LastLeftSelectedItemRow == self then self:SetSelected(true, true) end
    if LastLeftSelectedItemRow == self then return end
    -- deselect old item
    if LastLeftSelectedItemRow then
        LastLeftSelectedItemRow:SetSelected(false, true)
    end
    -- set selected for new item
    LastLeftSelectedItemRow = self

    -- clear old selected data
    OpeningFrame:ClearContentData()
    OpeningFrame:UpdateContentLists()
end

local function Opening_Statistics_OnOpeningItemUpdate(statistics, session, sourceID, itemKey)
    local sourceData = statistics:GetOpeningDataForSourceID(sourceID)
    if not sourceData then return end
    OpeningFrame:SetOpeningData(sourceID, itemKey, sourceData.itemCount, sourceData.itemValue)
end

local BOX_OPENING_FRAME_FUNCTIONS = {
    ["Clear"] = function(self, keepSession)
        if not keepSession then
            self:SetSession()
        end

        -- clear all lists
        self.itemContainer_left:ReleaseChildren()
        -- clear itemlist
        wipe(self.itemList)

        self:ClearContentData()
    end,

    ["ClearContentData"] = function(self)
        -- clear lists
        self.itemContainer_center:ReleaseChildren()
        --self.itemContainer_right:ReleaseChildren()
        wipe(self.contentList)
    end,

    ["SetSession"] = function(self, session)
        if session and session == self.session then
            return
        elseif self.session and session ~= self.session then
            if self.session.statistics then
                self.session.statistics:UnregisterCallbackEvent("Statistics_OnOpeningItemUpdate", Opening_Statistics_OnOpeningItemUpdate)
            end
            self:Clear(true)
            self.session = nil
        end
        self.session = session
        if not session then return end

        session.statistics:RegisterCallbackEvent("Statistics_OnOpeningItemUpdate", Opening_Statistics_OnOpeningItemUpdate)
    end,

    ["UpdateContentLists"] = function(self)
        if not LastLeftSelectedItemRow or not self.session or not self.session.statistics then return end
        local sourceID = LastLeftSelectedItemRow.item
        local sourceData = self.session.statistics:GetOpeningDataForSourceID(sourceID)
        if not sourceData then return end

        for i, itemKey in ipairs(sourceData.itemList) do
            local content = sourceData.content[itemKey]
            if content then
                self:SetContentData(sourceID, itemKey, content[1], content[2])
            end
        end

        self.itemContainer_center:SortChildrenList()
    end,

    ["AddOpeningItem"] = function(self, item)
        -- item exists in list or no item given
        if not item or self.itemList[item] then return self.itemList[item] end

        -- create a new row
        local row = AceGUI:Create("GoldGoblinLootGUIWidget-OpeningItem")
        row:SetFullWidth(true)
        row:SetItemData(item, 0, 0)
        row:SetToolTipPoint("BOTTOM_X", self.frame)
        row:SetCallback("SelectStateChange", LeftItemRow_SelectStateChange)
        self.itemContainer_left:AddChild(row)

        self.itemList[item] = {
            row = row,
            contentItems = {}
        }
        if not self.itemList.count then
            self.itemList.count = 1
            -- select first row if nothing is there
            row:SetSelected(true)
        else
            self.itemList.count = self.itemList.count + 1
        end

        return self.itemList[item], true
    end,

    ["SetOpeningData"] = function(self, sourceItem, contentItem, fullAmmount, fullValue)
        if not sourceItem or not contentItem then return end
        local row, isNewItem = self:AddOpeningItem(sourceItem)
        if not row then return end

        row.row:SetItemData(sourceItem, fullAmmount or 0, fullValue or 0)
        self:SetContentData(sourceItem, contentItem)
    end,


    ["AddContentItem"] = function(self, item)
        -- item exists in list or no item given
        if not item or self.contentList[item] then return self.contentList[item] end

        -- create a new row
        local row = AceGUI:Create("GoldGoblinLootGUIWidget-OpeningItem")
        row:SetFullWidth(true)
        row:SetItemData(item, 0, 0)
        row:SetToolTipPoint("BOTTOM_X", self.frame)
        --row:SetCallback("SelectStateChange", LeftItemRow_SelectStateChange)
        self.itemContainer_center:AddChild(row)

        self.contentList[item] = {
            row = row
        }
        if not self.contentList.count then
            self.contentList.count = 1
        else
            self.contentList.count = self.contentList.count + 1
        end

        return self.contentList[item], true
    end,

    ["SetContentData"] = function(self, sourceItem, contentItem, fullAmmount, fullValue)
        if not sourceItem or not contentItem then return end
        if not LastLeftSelectedItemRow or LastLeftSelectedItemRow.item ~= sourceItem then return end
        local row, isNewItem = self:AddContentItem(contentItem)
        if not row then return end

        if not fullAmmount then
            local sourceData = self.session.statistics:GetOpeningDataForSourceID(sourceItem)
            if not sourceData then return end
            sourceData = sourceData.content[contentItem]
            if not sourceData then return end
            fullAmmount, fullValue = sourceData[1], sourceData[2]
        end

        row.row:SetItemData(contentItem, fullAmmount or 0, fullValue or 0)
    end,

}

local function Create_Opening_Frame(self)
    if not self.openingFrame then
        local window = AceGUI:Create("Window")
        GGL.GUI_FixCloseButton(window.closebutton)
        window:SetTitle("GoldGoblinLoot - "..L["Opening"])

        local frame = window.frame
        local content = window.content

        -- some frame settings
        if frame.SetResizeBounds then
			frame:SetResizeBounds(OPENING_FRAME_DEFAULTS.MIN_WIDTH, OPENING_FRAME_DEFAULTS.MIN_HEIGHT,OPENING_FRAME_DEFAULTS.MAX_WIDTH, OPENING_FRAME_DEFAULTS.MAX_HEIGHT)
		else
            frame:SetMinResize(OPENING_FRAME_DEFAULTS.MIN_WIDTH, OPENING_FRAME_DEFAULTS.MIN_HEIGHT)
            frame:SetMaxResize(OPENING_FRAME_DEFAULTS.MAX_WIDTH, OPENING_FRAME_DEFAULTS.MAX_HEIGHT)
        end
        frame:SetClampedToScreen(true)
        frame:SetToplevel(false)
        frame:SetFrameStrata("MEDIUM")
        --window settings
        window:SetWidth(OPENING_FRAME_DEFAULTS.MIN_WIDTH)
        window:SetHeight(OPENING_FRAME_DEFAULTS.MIN_HEIGHT)
        -- remove resizer lines
        window.line1:Hide()
        window.line2:Hide()

        local NumBoxes = 2
        local function NewCatBox(name)
            --catBoxBG
            local catBoxBG = GGL.GUI:CreateBorderedFrame(content)
            -- catBox
            local catBox = AceGUI:Create("GoldGoblinLootGUIContainer-SortedScrollFrame")
            catBox.frame:SetParent(frame)
            catBox:SetWidth((content:GetWidth() - (OPENING_FRAME_DEFAULTS.BOX_GAP * (NumBoxes-1)) - (5*(NumBoxes-1))) / NumBoxes)
            catBox:SetPoint("BOTTOM", content, 0, 5)
            --catBox:SetDescending(true)
            -- catBox:SetCallback("BeforeListSort", print)
            catBox:SetSortFunction(OpeningSortFunction)
            window[name] = catBox
            -- catBox / bg
            catBoxBG:SetPoint("TOPLEFT", catBox.scrollframe, -5, 5)
            catBoxBG:SetPoint("BOTTOMRIGHT", catBox.scrollframe, 5, -5)
            return catBox
        end

        local itemContainer_left = NewCatBox("itemContainer_left")
        itemContainer_left:SetPoint("TOPLEFT", content, "TOPLEFT", 0, 0)
        --itemContainer_left:SetCallback("BeforeListSort", ItemContainerLeft_BeforeListSort)
        itemContainer_left:SetRefreshInterval(OPENING_SOURCE_LIST_REFRESH_INTERVAL)

        local itemContainer_center = NewCatBox("itemContainer_center")
        itemContainer_center:SetPoint("TOPLEFT", window.itemContainer_left.frame, "TOPRIGHT", OPENING_FRAME_DEFAULTS.BOX_GAP, 0)
        itemContainer_center:SetRefreshInterval(OPENING_SOURCE_LIST_REFRESH_INTERVAL)

        --local itemContainer_right = NewCatBox("itemContainer_right")
        --itemContainer_right:SetPoint("TOPLEFT", window.itemContainer_center.frame, "TOPRIGHT", OPENING_FRAME_DEFAULTS.BOX_GAP, 0)

        -- var
        window.itemList = {}
        window.contentList = {}

        -- register functions
        for k,v in pairs(BOX_OPENING_FRAME_FUNCTIONS) do
            window[k] = v
        end
        BOX_OPENING_FRAME_FUNCTIONS = nil

        self.openingFrame = window
        OpeningFrame = window
    end
    return self.openingFrame
end

function SpecialEvent:ShowOpening()
    Create_Opening_Frame(self):Show()
end

function SpecialEvent:GetOpeningFrame()
    return Create_Opening_Frame(self)
end

function SpecialEvent:AddOpeningIntoMiniMapButton()
    GGL:RegisterMMFunction(self, self.ShowOpening)
end

function SpecialEvent:RemoveOpeningFromMiniMapButton()
    if not self.ShowOpening then return end
    GGL:UnregisterMMFunction(self.ShowOpening)
end

function SpecialEvent:RemoveAllFromFromMiniMapButton()
    self:RemoveOpeningFromMiniMapButton()
end

function SpecialEvent:RemoveAllEvents()
    SpecialEvent:RemoveAllFromFromMiniMapButton()
    if self.ShowOpening then
        self.ShowOpening:SetSession()
    end
end
