local AddonName, GGL = ...

local GUI = {}
GGL.GUI = GUI

local L = GGL.Locale

local AceGUI = LibStub("AceGUI-3.0")

local GetScreenHeight, GetScreenWidth = GetScreenHeight, GetScreenWidth

function GUI.Init()

end
--GGL:AddInitFuncPlayerLogin(GUI.Init)

-- ########################################
-- Line
-- ########################################
function GUI.CreateLine(parent, isVertical, point1, point2, width, thickness)
    local line = parent:CreateTexture(nil, "BACKGROUND")
    thickness = thickness or 2

    if isVertical then
        if width then
            line:SetHeight(width)
        end
        line:SetWidth(thickness)
    else
        line:SetHeight(thickness)
        if width then
            line:SetWidth(width)
        end
    end
    line:SetPoint(point1[1], point1[2] or parent, point1[3], point1[4], point1[5])
    line:SetPoint(point2[1], point2[2] or parent, point2[3], point2[4], point2[5])

    line:SetColorTexture(1, 1, 1, 0.5)

    return line
end

-- ########################################
-- Frames
-- ########################################
local BorderedFrameBackdrop  = {
	bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
	tile = true, tileSize = 16, edgeSize = 16,
	insets = { left = 3, right = 3, top = 5, bottom = 3 }
}

local BoxFrameBackdrop  = {
    bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
    edgeFile = "Interface\\Buttons\\WHITE8X8",
    tile = false,
    edgeSize = 1,
	--insets = { left = 0, right = 0, top = 0, bottom = 0 }
}

--- Creates an bordered frame
-- @param       frame       <UIFrame> parent
function GUI:CreateBorderedFrame(frame)
    local bg = CreateFrame("Frame", nil, frame, _G.BackdropTemplateMixin and "BackdropTemplate" or nil)

    bg:SetBackdrop(BorderedFrameBackdrop)
	bg:SetBackdropColor(0.1, 0.1, 0.1, 0.5)
	bg:SetBackdropBorderColor(0.4, 0.4, 0.4)

    return bg
end

--- Creates an box frame
-- @param       frame       <UIFrame> parent
function GUI:CreateBoxFrame(frame)
    local bg = CreateFrame("Frame", nil, frame, _G.BackdropTemplateMixin and "BackdropTemplate" or nil)

    bg:SetBackdrop(BoxFrameBackdrop)
	bg:SetBackdropColor(0, 0, 0, 0.5)
	bg:SetBackdropBorderColor(0, 0, 0, 0.8)

    return bg
end

local function BoxButton_OnEnter(self)
    self.bg:SetBackdropBorderColor(0, 112, 222, 0.6)
    if self.OnEnter then
        self.OnEnter(self)
    end
end

local function BoxButton_OnLeave(self)
    self.bg:SetBackdropBorderColor(1, 1, 1, 0.6)
    if self.OnLeave then
        self.OnLeave(self)
    end
end

--- Creates an boxed button
-- @param       frame       <UIFrame> parent
function GUI:CreateBoxButton(parFrame)
    local button = CreateFrame("BUTTON", nil, parFrame)
    button:SetScript("OnEnter", BoxButton_OnEnter)
    button:SetScript("OnLeave", BoxButton_OnLeave)
    button:SetHeight(20)

    local bg = GUI:CreateBoxFrame(button)
    bg:SetAllPoints(button)
    bg:SetBackdropBorderColor(1, 1, 1, 0.6)
    button.bg = bg

    local text = bg:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmall")
    text:SetAllPoints(button)
    text:SetJustifyH("CENTER")
    text:SetJustifyV("MIDDLE")
    text:SetNonSpaceWrap(false)
    button.text = text

    return button
end

-- ########################################
-- Tooltip
-- ########################################

-- Tooltip
local ToolTipPoints = {
    -- top LEFT/RIGHT from origin frame
    ["TOP_X"] = function(overTop, isLeft, ownerFrame, xPoint, yPoint)
        local fPoint, oPoint = "TOP", "TOP"
        if isLeft then
            fPoint, oPoint = fPoint.."LEFT", oPoint.."RIGHT"
        else
            fPoint, oPoint = fPoint.."RIGHT", oPoint.."LEFT"
            xPoint = xPoint and (-xPoint) or nil
        end
        return fPoint, ownerFrame, oPoint, xPoint, yPoint
    end,

    -- bottom LEFT/RIGHT from origin frame
    ["BOTTOM_X"] = function(overTop, isLeft, ownerFrame, xPoint, yPoint)
        local fPoint, oPoint = "BOTTOM", "BOTTOM"
        if isLeft then
            fPoint, oPoint = fPoint.."RIGHT", oPoint.."LEFT"
            xPoint = xPoint and (-xPoint) or nil
        else
            fPoint, oPoint = fPoint.."LEFT", oPoint.."RIGHT"
        end
        return fPoint, ownerFrame, oPoint, xPoint, yPoint
    end,

    -- center LEFT/RIGHT from origin frame
    ["X"] = function(overTop, isLeft, ownerFrame, xPoint, yPoint)
        local fPoint, oPoint
        if isLeft then
            fPoint, oPoint = "LEFT", "RIGHT"
        else
            fPoint, oPoint = "RIGHT", "LEFT"
            xPoint = xPoint and (-xPoint) or nil
        end
        return fPoint, ownerFrame, oPoint, xPoint, yPoint
    end,

    -- center TOP/BOTTOM from origin frame
    ["Y"] = function(overTop, isLeft, ownerFrame, xPoint, yPoint)
        local fPoint, oPoint
        if overTop then
            fPoint, oPoint = "TOP", "BOTTOM"
        else
            fPoint, oPoint = "BOTTOM", "TOP"
            yPoint = yPoint and (-yPoint) or nil
            xPoint = xPoint or 0
        end
        return fPoint, ownerFrame, oPoint, xPoint, yPoint
    end,
}

--- Set Point for ToolTip
-- @param       ttFrame         <GameTooltip>
-- @param       originFrame     <Frame>
-- @param       originPoint     <string>
-- @param       xPoint          <number>
-- @param       yPoint          <number>
-- @return oriantationX ("LEFT", "RIGHT"), oriantationY ("TOP", "BOTTOM")
function GUI:SetToolTipPoint(ttFrame, ownerFrame, originPoint, xPoint, yPoint)
    ttFrame = ttFrame or GameTooltip
    if not ToolTipPoints[originPoint] then return end

    -- Get center from tooltip frame
    local x,y = ownerFrame:GetCenter()

    -- Set tt point
    --ttFrame:ClearAllPoints()
    ttFrame:SetOwner(ownerFrame,"ANCHOR_NONE")
    ttFrame:SetPoint( ToolTipPoints[originPoint](y/GetScreenHeight() > 0.3, x/GetScreenWidth() > 0.5, ownerFrame, xPoint, yPoint) )

    return ((x/GetScreenWidth() > 0.5) and "LEFT" or "RIGHT"), ((y/GetScreenHeight() > 0.3) and "TOP" or "BOTTOM")
end

--- Set Point for ToolTip Frames
-- @param       ttFrame         <GameTooltip>
-- @param       originFrame     <Frame>
-- @param       originPoint     <string>
-- @param       xPoint          <number>
-- @param       yPoint          <number>
-- @return oriantationX ("LEFT", "RIGHT"), oriantationY ("TOP", "BOTTOM")
function GUI:SetToolTipPoint_Frame(ttFrame, ownerFrame, originPoint, xPoint, yPoint)
    ttFrame = ttFrame or GameTooltip
    if not ToolTipPoints[originPoint] then return end

    -- Get center from tooltip frame
    local x,y = ownerFrame:GetCenter()

    -- Set tt point
    ttFrame:ClearAllPoints()
    ttFrame:SetParent(ownerFrame)
    ttFrame:SetPoint( ToolTipPoints[originPoint](y/GetScreenHeight() > 0.3, x/GetScreenWidth() > 0.5, ownerFrame, xPoint, yPoint) )

    return ((x/GetScreenWidth() > 0.5) and "LEFT" or "RIGHT"), ((y/GetScreenHeight() > 0.3) and "TOP" or "BOTTOM")
end

-- OnEnter cb
local function AceToolTip_OnEnter(self)
    local tooltip = _G.GameTooltip
    tooltip:ClearLines()
    tooltip:SetOwner(self.frame, "ANCHOR_RIGHT")
    tooltip:AddLine(self:GetUserData("title"))
    local text = self:GetUserData("text")
    for i = 1, #text do
        tooltip:AddLine(text[i], 1, 1, 1)
    end
    tooltip:Show()
end

-- OnLeave cb
local function AceToolTip_OnLeave(self)
    _G.GameTooltip:Hide()
end

--- Adds a Tooltip for a AceGUI widget
-- only works with widgets that have the "OnEnter" and "OnLeave" cb's and an widget.frame
-- @param       widget      <AceGUI widget>
-- @param       title       <string> the title text
-- @param       ...         <string's> the text
function GUI:AddToolTipForAceGUIWidget(widget, title, ...)
    if not widget or not widget.frame then return end
    -- set empty strings of none are given
    title = title or ""

    -- set user objects
    widget:SetUserData("title", title)
    widget:SetUserData("text", {...})

    -- register callbacks
    widget:SetCallback("OnEnter", AceToolTip_OnEnter)
    widget:SetCallback("OnLeave", AceToolTip_OnLeave)
end

-- ########################################
-- Frame control
-- ########################################
local AddFrameControl, ResetAllControlFrames
do
    local SCALE_STEP = 0.1

    local ControlFrameRegister = {}

    local function ControlGetDBValue(self, key)
        return ControlFrameRegister[self].db[key]
    end

    local function ControlSetDBValue(self, key, value)
        ControlFrameRegister[self].db[key] = value
    end

    local function SetFramePosition(self)
        local points = ControlGetDBValue(self, "points") or {"CENTER"}
        self:ClearAllPoints()
        self:SetPoint(points[1], points[2] or UIParent, points[3], points[4], points[5])
        if ControlGetDBValue(self, "width") then
            self:SetWidth(ControlGetDBValue(self, "width"))
        end
        if ControlGetDBValue(self, "height") then
            self:SetHeight(ControlGetDBValue(self, "height"))
        end
    end

    -- TODO: #7 Set Position on top after scaling with FrameControl
    local function SetFrameScale(self)
        self:SetScale(ControlGetDBValue(self, "scale") or 1.0)
    end

    -- delta: 1=up, -1=down
    local function FrameOnMouseWheel(self, delta)
        if not ControlFrameRegister[self] or not ControlFrameRegister[self].isScaleable then return end
        if IsAltKeyDown() then
            local scale = self:GetScale()
            if delta == -1 then
                scale = max(scale-SCALE_STEP, 0.5)
            elseif delta == 1 then
                scale = min(scale+SCALE_STEP, 2.0)
            end
            ControlSetDBValue(self, "scale", scale)
            SetFrameScale(self)
        end
    end

    local function FrameOnDragStop(self)
        self:__StopMovingOrSizing()
        if not ControlFrameRegister[self] or not ControlFrameRegister[self].isMoveable then return end
        local point, relativeTo, relativePoint, xOfs, yOfs = self:GetPoint()

        ControlSetDBValue(self, "points", {point, nil, relativePoint, xOfs, yOfs})
        ControlSetDBValue(self, "width", self:GetWidth())
        ControlSetDBValue(self, "height", self:GetHeight())
    end

    function AddFrameControl(name, frame, db, isMoveable, isScaleable)
        if ControlFrameRegister[frame] then return end

        ControlFrameRegister[frame] = {
            name = name,
            isMoveable = isMoveable,
            isScaleable = isScaleable,
            db = db,
        }

        -- small workaround for the moving
        if isMoveable then
            frame.__StopMovingOrSizing = frame.StopMovingOrSizing
            frame.StopMovingOrSizing = FrameOnDragStop
        end

        if isScaleable then
            frame:HookScript("OnMouseWheel", FrameOnMouseWheel)
        end
        --frame:HookScript("OnDragStop", FrameOnDragStop)

        -- setup from db
        SetFramePosition(frame)
        SetFrameScale(frame)
    end

    function ResetAllControlFrames()
        for frame, control in pairs(ControlFrameRegister) do
            if control.isMoveable then
                ControlSetDBValue(frame, "points", nil)
                SetFramePosition(frame)
            end

            if control.isScaleable then
                ControlSetDBValue(frame, "scale", nil)
                SetFrameScale(frame)
            end
        end
    end
end

--- Adds some controls for the frame
-- position and scaling size save and make frame scaleabel
-- @parm        name            <string> a unique name for the frame
-- @parm        frame           <ACE-GUI-Window>
-- @parm        isMoveable      <bool> this only saves position and not make the frame moveable
-- @parm        isScaleable     <bool> make frame scaleable and saves it
function GUI:AddFrameControl(name, frame, isMoveable, isScaleable)
    if not name or not frame then return end

    -- get db
    local db = GGL.db.Settings.FrameControl[name]
    if not db then
        GGL.db.Settings.FrameControl[name] = {}
        db = GGL.db.Settings.FrameControl[name]
    end

    AddFrameControl(name, frame.frame or frame, db, isMoveable, isScaleable)
end

--- Resets all frames
-- Position and scaling
function GUI:ResetAllControlFrames()
    ResetAllControlFrames()
end

-- ########################################
-- Reload progress bar frame
-- ########################################
local function RebuildFrame_OnClose(window)
    window.loaded = false
end

function GUI:RebuildFrame_Get()
    if not self.rebuildFrame then
        local frameWidth, frameHeight = 250, 70
        local window = AceGUI:Create("Window")
        GGL.GUI_FixCloseButton(window.closebutton)
        window:SetTitle("GoldGoblinLoot - "..L["Rebuild"])
        window:SetCallback("OnClose", RebuildFrame_OnClose)

        local frame = window.frame
        local content = window.content

        -- some frame settings
        if frame.SetResizeBounds then
            frame:SetResizeBounds(frameWidth, frameHeight, frameWidth, frameHeight)
        else
            frame:SetMinResize(frameWidth, frameHeight)
            frame:SetMaxResize(frameWidth, frameHeight)
        end
        frame:SetClampedToScreen(true)
        frame:SetToplevel(false)
        frame:SetFrameStrata("MEDIUM")
        --window settings
        window:SetLayout("Flow")
        window:SetWidth(frameWidth)
        window:SetHeight(frameHeight)
        -- remove resizer lines
        window.line1:Hide()
        window.line2:Hide()

        local bar = AceGUI:Create("GoldGoblinLootGUIWidget-ProgressBar")
        bar:SetFullWidth(true)
        bar:SetBarColor(0, 1, 0)
        bar:SetPercent(true,false)
        window.bar = bar
        window:AddChild(bar)

        local bgFrame = GUI:CreateBoxFrame(bar.frame)
        bgFrame:SetPoint("TOPLEFT", bar.frame, -2, 1)
        bgFrame:SetPoint("BOTTOMRIGHT", bar.frame, 2, -1)

        self.rebuildFrame = window
    end
    return self.rebuildFrame
end

function GUI:RebuildFrame_Add(state)
    if state then
        local rebuildFrame = GUI:RebuildFrame_Get()
        rebuildFrame:Show()
    elseif self.rebuildFrame then
        self.rebuildFrame:Hide()
    end
    if self.rebuildFrame then
        self.rebuildFrame.loaded = state
    end
end

function GUI:RebuildFrame_Set(minValue, maxValue, curValue)
    if self.rebuildFrame and self.rebuildFrame.loaded then
        self.rebuildFrame.bar:SetValue(minValue, maxValue, curValue)
        if maxValue == curValue then
            GUI:RebuildFrame_Add(false)
        end
    end
end

-- ##################
-- Layout
-- ##################
local function safecall(func, ...)
	if func then
		return xpcall(func, error, ...)
	end
end

AceGUI:RegisterLayout("GGL-Down2Up",
    function(content, children)
        local height = 0
        local width = content.width or content:GetWidth() or 0
        for i = 1, #children do
            local child = children[i]

            local frame = child.frame
            frame:ClearAllPoints()
            frame:Show()
            if i == 1 then
                frame:SetPoint("BOTTOMLEFT", content)
            else
                children[i-1].frame:SetPoint("BOTTOMLEFT", frame, "TOPLEFT")
                frame:SetPoint("BOTTOMLEFT", content)
            end

            if child.width == "fill" then
                child:SetWidth(width)
                frame:SetPoint("RIGHT", content)

                if child.DoLayout then
                    child:DoLayout()
                end
            elseif child.width == "relative" then
                child:SetWidth(width * child.relWidth)

                if child.DoLayout then
                    child:DoLayout()
                end
            end

            height = height + (frame.height or frame:GetHeight() or 0)
        end
        safecall(content.obj.LayoutFinished, content.obj, nil, height)
    end)

AceGUI:RegisterLayout("GGL-ListWithGap",
	function(content, children)
		local height = 0
        local width = content.width or content:GetWidth() or 0
		for i = 1, #children do
            local child = children[i]
            local gap = child:GetUserData("listGap")

			local frame = child.frame
			frame:ClearAllPoints()
			frame:Show()
			if i == 1 then
				frame:SetPoint("TOPLEFT", content)
			else
				frame:SetPoint("TOPLEFT", children[i-1].frame, "BOTTOMLEFT", 0, -(gap))
			end

			if child.width == "fill" then
				child:SetWidth(width)
				frame:SetPoint("RIGHT", content)

				if child.DoLayout then
					child:DoLayout()
				end
			elseif child.width == "relative" then
				child:SetWidth(width * child.relWidth)

				if child.DoLayout then
					child:DoLayout()
				end
			end

			height = height + (frame.height or frame:GetHeight() or 0) + (i == 1 and 0 or gap)
		end
		safecall(content.obj.LayoutFinished, content.obj, nil, height)
	end)