local AddonName, GGL = ...
local GUI = GGL.GUI
local Global = {}
GGL.Options.Extra = Global

local L = GGL.Locale

-- lua

-- WoW

-- Libs
local AceGUI = LibStub("AceGUI-3.0")
local Archivist

-- ##################
-- local
-- ##################
local SessionRegister = {}
local ArchiveFrame

local function WidgetOnRelease(self)
    ArchiveFrame = nil
    wipe(SessionRegister)
end

local function ArchiveList_AfterListSort(self)

end

-- ##################
-- local helper
-- ##################
local function RemoveEntryFromListAndDB(sessionLine)
    if not ArchiveFrame then return end
    Archivist:Delete("ReadOnly", sessionLine.sessionID, true)
    ArchiveFrame:ReleaseSingleChildren(sessionLine)
    ArchiveFrame:SortChildrenList()
    SessionRegister[sessionLine] = nil
end

local function SessionLine_DeleteOnClick(self, event, mb)
    if IsShiftKeyDown() then
        RemoveEntryFromListAndDB(self)
    else
        GGL.Options:OpenConfirm(
            format(L["Delete session '|cffff0000%s|r'?"], self.sessionID),
            RemoveEntryFromListAndDB,
            nil,
            self
        )
    end
end

local function ExportButton_OnClick(self, event, mb)
    if GoldGoblinLootDataDB then
        local exportTable = {
            locale = GoldGoblinLootDataDB.Locale,
            npc_names = GoldGoblinLootDataDB.npc_names,
            users = {}
        }
        for playerSyncID, playerData in pairs(GGL.db.PlayerRegister) do
            if playerSyncID and playerData.info and playerData.charName then
                exportTable.users[#exportTable.users+1] = {playerSyncID, playerData.charName, playerData.info}
            end
        end
        local exportString = GGL.GetCompressedString(exportTable)
        GGL.Export:FillExportWindow("ggldata:"..(exportString or ""), "text")
    end
end


-- ##################
-- ArchiveOptions
-- ##################
local function ArchiveOptions(AceGUI, container)

    local scroll = AceGUI:Create("GoldGoblinLootGUIContainer-SortedScrollFrame")
    scroll:SetFullWidth(true)
    scroll:SetCallback("AfterListSort", ArchiveList_AfterListSort)
    scroll:SetCallback("OnRelease", WidgetOnRelease)
    container:AddChild(scroll)

    -- fill list
    if not IsAddOnLoaded("GoldGoblinLootArchive") then
        local ok, reason = LoadAddOn("GoldGoblinLootArchive")
        if not ok then
            GGL:AddIntoErrorLog("AddSessionIntoArchive", reason)
        end
    end
    if not Archivist and GGL.Archivist and GGL.Archivist.Initialize then
        Archivist = GGL.Archivist:Initialize(GoldGoblinLootArchiveDB)
    elseif not Archivist then
        GGL:AddIntoErrorLog("AddSessionIntoArchive", "Archivist full missing")
    end
    if Archivist and Archivist.Check then
        local clearList = {}
        for sessionID, sessionInfo in pairs(GoldGoblinLootArchiveDataDB) do
            if Archivist:Check("ReadOnly", sessionID) then
                -- session Found
                local sessionLine = AceGUI:Create("GoldGoblinLootGUIWidget-ArchiveEntry")
                sessionLine:SetFullWidth(true)
                sessionLine:SetArchiveData(Archivist, sessionID, sessionInfo)
                sessionLine:SetCallback("DeleteOnClick", SessionLine_DeleteOnClick)
                scroll:AddChild(sessionLine)
                SessionRegister[sessionID] = sessionLine
            else
                -- session not found
                clearList[#clearList+1] = sessionID
            end
        end

        -- clear not found entrys
        if #clearList > 0 then
            for i = 1, #clearList do
                GoldGoblinLootArchiveDataDB[clearList[i]] = nil
            end
        end

        -- sort list
        scroll:SortChildrenList()
    else
        GGL:AddIntoErrorLog("AddSessionIntoArchive", "Archivist init missing")
    end

    ArchiveFrame = scroll
end

-- ##################
-- ArchiveOptions
-- ##################
local function ExportOptions(AceGUI, container)

    local scroll = AceGUI:Create("ScrollFrame")
    scroll:SetLayout("Flow")
    scroll:SetCallback("OnRelease", WidgetOnRelease)
    container:AddChild(scroll)

    local exportButton = AceGUI:Create("Button")
    exportButton:SetText(L["Export"])
    exportButton:SetFullWidth(true)
    exportButton:SetCallback("OnClick", ExportButton_OnClick)
    scroll:AddChild(exportButton)
end

-- Add into Options
GGL.Options:AddDisabledIntoTree(nil, L["Extra Settings"], "addonSettings")
GGL.Options:AddIntoTree(ArchiveOptions, L["Archive"], "archiveData")
GGL.Options:AddIntoTree(ExportOptions, L["Export"], "exportData")