-- ##################
--- EVENTS ( GGL.Templates.BNWoWPlayer:RegisterCallbackEvent )
-- "PlayerTemplate_NewPlayerRegistered" (playerTemplate)
-- "PlayerTemplate_PlayerUpdate" (playerTemplate)
-- "PlayerTemplate_DisplayNameUpdate" (playerTemplate, displayName)
-- "PlayerTemplate_AccountIDUpdate" (playerTemplate, newAccountID, oldAccountID)
-- "PlayerTemplate_MergeIDUpdate" (playerTemplate, newMergeID, oldMergeID)
-- "PlayerTemplate_RewardIDUpdate" (playerTemplate, rewardID)
-- ##################

local AddonName, GGL = ...
local Templates = GGL.Templates

local BNWoWPlayer, BNWoWPlayerHelper = Templates:Register("BNWoWPlayer")

-- lua
local assert, type = assert, type
local pairs, next = pairs, next
local match, format = string.match, string.format

local C_BattleNet = C_BattleNet or GGL.C_BattleNet

-- wow
local BNGetGameAccountInfoByGUID, BNGetAccountInfoByGUID, BNGetGameAccountInfoByID = C_BattleNet.GetGameAccountInfoByGUID, C_BattleNet.GetAccountInfoByGUID, C_BattleNet.GetGameAccountInfoByID

local WOW_PROJECT_ID = _G.WOW_PROJECT_ID

-- ##################
-- local
-- ##################
local MERGE_ID_FORMAT = "%s!%d"

local DummyGUIDRegister = {}
local GUIDRegister = {}
local GameAccountIDRegister = {}
local SpecialRegister = {}
local PlayerSelf

-- ##################
-- static
-- ##################
local Static = {}
Templates.BNWoWPlayer = Static

function Static:Get(unitGUID)
    return unitGUID and GUIDRegister[unitGUID] or nil
end

function Static:GetByGameAccountID(gameAccountID)
    if gameAccountID and GameAccountIDRegister[gameAccountID] then
        return GameAccountIDRegister[gameAccountID]
    end
end

function Static:SetOnlineByGameAccountID(gameAccountID)
    if gameAccountID and GameAccountIDRegister[gameAccountID] then
        GameAccountIDRegister[gameAccountID]:SetOnline(true)
    end
end

function Static:SetGGLAviableByGameAccountID(gameAccountID)
    if gameAccountID and GameAccountIDRegister[gameAccountID] and not GameAccountIDRegister[gameAccountID].gglVersion then
        GameAccountIDRegister[gameAccountID].gglVersion = GGL.UNKNOWN_GGL_VERSION
    end
end

function Static:GetByGUID(unitGUID)
    if unitGUID and GUIDRegister[unitGUID] then
        return GUIDRegister[unitGUID]
    end
end

function Static:GetPlayerSender()
    if not self.playerSender then
        local sender = C_BattleNet.GetGameAccountInfoByGUID(UnitGUID("player"))
        if sender then
            self.playerSender = sender.gameAccountID
        end
    end
    return self.playerSender
end

-- method(BNWoWPlayer)
function Static:RegisterCallbackEvent(...)
    BNWoWPlayerHelper:RegisterCallbackEvent(...)
end

function Static:UnregisterCallbackEvent(...)
    BNWoWPlayerHelper:UnregisterCallbackEvent(...)
end

-- for playerGUID, player in Templates.BNWoWPlayer:IterPlayerList() do end
function Static:IterPlayerList()
    return next, GUIDRegister, nil
end

local function EmptyDummyFunction() end

function Static:GetDummyPlayer(syncID)
    if DummyGUIDRegister[syncID] then
        return DummyGUIDRegister[syncID]
    else
        local battleTag, unitGUID = match(syncID, "^(.*):(.*)")
        if not battleTag then return GGL:AddIntoErrorLog("BNWoWPlayer / Static:GetDummyPlayer", syncID) end

        -- create a new player entry if its not existing or we get errors
        if not GGL.db.PlayerRegister[syncID] then
            GGL.db.PlayerRegister[syncID] = {}
        end

        local _, ownBattleTag = BNGetInfo()
        local playerGUID = UnitGUID("player")

        local dummy = {
            -- block some calls here
            Constructor = EmptyDummyFunction,
            Update = EmptyDummyFunction,
            RegisterCallbackEvent = EmptyDummyFunction,
            UnregisterCallbackEvent = EmptyDummyFunction,
            FireCallbackEvent = EmptyDummyFunction,
            -- some needed things
            characterName = GGL.db.PlayerRegister[syncID].charName,
            syncID = syncID,
            gglVersion = GGL.UNKNOWN_GGL_VERSION,
            isOnline = false,
            unitGUID = unitGUID,
            isPlayerBattleTag = ownBattleTag == battleTag,
            isPlayer = playerGUID == unitGUID,
            bnPlayer = {
                battleTag = battleTag,
            },
            displayName = GGL.db.PlayerRegister[syncID].displayName or syncID,
            -- detection that its a dummy
            isDummy = true,
        }
        if GGL.db.PlayerRegister[syncID].info then
            local realmID, factionID, raceID, classID = match(GGL.db.PlayerRegister[syncID].info, "(%d+):(%d+):(%d+):(%d+)")
            self.realmID, factionID, raceID, classID = tonumber(realmID) or 0, tonumber(factionID) or 0, tonumber(raceID) or 0, tonumber(classID) or 0
            dummy.realmName = GoldGoblinLootDataDB["realm_names"][self.realmID] or "realmID:"..realmID
            dummy.factionName = GGL.FACTION_ID_TO_NAME[factionID]
            dummy.raceName = GGL.RACE_ID_REGISTER[raceID] and GGL.RACE_ID_REGISTER[raceID].raceName or nil
            dummy.className = _G.LOCALIZED_CLASS_NAMES_MALE[GGL.CLASS_ID_TO_NAME[classID]]
        end
        setmetatable(dummy, {__index = BNWoWPlayer})
        dummy:UpdateMergeID()
        DummyGUIDRegister[syncID] = dummy

        return dummy
    end
end

-- Creates a template of the player
function Static:GetSelfPlayer()
    if not PlayerSelf then
        if not Templates.BNPlayer:GetSelf() then return GGL:AddIntoErrorLog("BNWoWPlayer / Static:GetSelfPlayer", "NPlayer:GetSelf missing") end
        PlayerSelf = Templates:New("BNWoWPlayer", UnitGUID("player"), Templates.BNPlayer:GetSelf())
        PlayerSelf:SetDisplayName(GGL.db.Settings.Main.displayName)
    end
    return PlayerSelf
end

-- ##################
-- template
-- ##################
function BNWoWPlayer:Constructor(unitGUID, bnPlayer)
    assert(unitGUID, "No 'unitGUID' found ("..(unitGUID or "nil")..")")
    assert(bnPlayer, "No 'bnPlayer' found.")
    if GUIDRegister[unitGUID] then
        return GUIDRegister[unitGUID]
    end

    local isPlayer
    local playerGUID = UnitGUID("player")
    --check if guid is players guid
    if playerGUID == unitGUID then
        isPlayer = true
    end

    local bnetAccount = BNGetAccountInfoByGUID(unitGUID)
    -- maybe support non Bnetaccs later?
    if not isPlayer and not bnetAccount then return false end

    local gameAccount = BNGetGameAccountInfoByGUID(unitGUID)
    if not isPlayer and not gameAccount then return false end

    -- check if given can be used
    if isPlayer and bnetAccount and gameAccount then
        isPlayer = false
    end

    local _, ownBattleTag = BNGetInfo()

    -- battle.net account
    self.unitGUID = unitGUID
    self.bnetAccountID = isPlayer and (bnetAccount and bnetAccount.bnetAccountID or 0) or bnetAccount.bnetAccountID

    -- game account
    self.gameAccountID = isPlayer and (gameAccount and gameAccount.gameAccountID or 0) or gameAccount.gameAccountID
    -- self.realmName = gameAccount.realmName -- gets name later in game... use realmID
    self.isPlayer = playerGUID == unitGUID
    self.isPlayerBattleTag = ownBattleTag == bnPlayer.battleTag
    self.realmID = isPlayer and GetRealmID() or gameAccount.realmID
    self.realmID = self.realmID or 0
    self.characterName = isPlayer and UnitName("player") or gameAccount.characterName
    self.factionName = isPlayer and UnitFactionGroup("player") or gameAccount.factionName
    self.raceName = isPlayer and UnitRace("player") or gameAccount.raceName
    self.className = isPlayer and UnitClass("player") or gameAccount.className
    self.bnPlayer = bnPlayer
    self.battleTag = bnPlayer.battleTag
    self.syncID = bnPlayer.battleTag..":"..unitGUID
    self.sender = self.gameAccountID
    self.wowProjectID = gameAccount and gameAccount.wowProjectID or false
    self.isSameWowProjekt = WOW_PROJECT_ID == self.wowProjectID

    GUIDRegister[unitGUID] = self

    if not GGL.db.PlayerRegister[self.syncID] then
        GGL.db.PlayerRegister[self.syncID] = {}
    end

    --if not GGL.db.PlayerRegister[self.syncID].info then
        local classID = GGL.CLASS_NAME_TO_ID[GGL.LOCALIZED_CLASS_TO_NAME[self.className]]
        local factionID = GGL.FACTION_NAME_TO_ID[self.factionName]
        local raceID = GGL.RACE_NAME_TO_ID[self.raceName]
        GGL.db.PlayerRegister[self.syncID].info = format("%d:%d:%d:%d", self.realmID, factionID, raceID, classID)
    --end

    -- save char name
    GGL.db.PlayerRegister[self.syncID].charName = self.characterName

    -- save wow version if its not retail
    if self.wowProjectID and self.wowProjectID ~= WOW_PROJECT_MAINLINE then
        GGL.db.PlayerRegister[self.syncID].wowProjectID = self.wowProjectID
    end

    -- remove dummy if its there
    DummyGUIDRegister[self.syncID] = nil

    -- add own callback handler
    GGL:AddCallbackHandler(self)

    self:UpdateMergeID()
    self:Update()

    BNWoWPlayerHelper:FireCallbackEvent("PlayerTemplate_NewPlayerRegistered", self)
end

function BNWoWPlayer:Update()
    local gameAccount = BNGetGameAccountInfoByGUID(self.unitGUID)

    if self.isPlayer and not gameAccount then
        self.isOnline = true
    elseif not gameAccount then
        -- offline
        self.isOnline = false
    else
        if gameAccount then
            if self.isPlayer then
                self.isGameAFK = false
            else
                self.isGameAFK = gameAccount.isGameAFK
            end

            GameAccountIDRegister[self.gameAccountID] = self

            self.richPresence = gameAccount.richPresence -- "Dazar'alor - Blackrock"
            self.characterLevel = gameAccount.characterLevel
            self.areaName = gameAccount.areaName
            self.isWowMobile = gameAccount.isWowMobile
            self.isOnline = gameAccount.isOnline
            self.realmName = gameAccount.realmName or "realmID:"..self.realmID -- realmName gets loaded later after login but is there after a reload/refresh
            if GoldGoblinLootDataDB and GoldGoblinLootDataDB["realm_names"] then
                if gameAccount.realmName then
                    GoldGoblinLootDataDB["realm_names"][self.realmID] = gameAccount.realmName
                end
                self.realmName = GoldGoblinLootDataDB["realm_names"][self.realmID] or "realmID:"..self.realmID
            end
        end
        self.fullName = self.characterName.."-"..self.realmName

    end
    self:RefreshDisplayName()

    self:FireCallbackEvent("PlayerTemplate_PlayerUpdate", self)
    BNWoWPlayerHelper:FireCallbackEvent("PlayerTemplate_PlayerUpdate", self)
end

--- Updates the displayName for the player
function BNWoWPlayer:RefreshDisplayName()
    self.displayName = GGL.db.PlayerRegister[self.syncID].displayName or self.bnPlayer.battleTag or self.fullName
end

--- Sets the display name for a player
-- @param       displayName         <string>
function BNWoWPlayer:SetDisplayName(displayName)
    displayName = displayName and GGL.FormatDisplayName(displayName) or nil

    if not displayName or displayName == "" then
        -- set default to battle.tag
        displayName = self.bnPlayer.battleTag
    end

    -- update db for that player
    GGL.db.PlayerRegister[self.syncID].displayName = displayName == "" and nil or displayName

    -- update mergeID name
    if self.mergeID then
        GGL.db.MergeIDNames = GGL.db.PlayerRegister[self.syncID].displayName
    end

    -- update player
    self:Update()

    -- callbacks
    self:FireCallbackEvent("PlayerTemplate_DisplayNameUpdate", self, self:GetDisplayName())
    BNWoWPlayerHelper:FireCallbackEvent("PlayerTemplate_DisplayNameUpdate", self, self:GetDisplayName())
end

--- Returns displayName for a player
function BNWoWPlayer:GetDisplayName()
    return GGL.db.PlayerRegister[self.syncID].displayName or self.displayName or self.syncID
end

function BNWoWPlayer:SetGGRewardID(id)
    id = tonumber(id)
    if not id or id < 0 or id > 2 then
        id = nil
    else
        id = id
    end

    -- update db for that player
    GGL.db.PlayerRegister[self.syncID].rewardID = id

    -- RewardIDUpdate
    self:FireCallbackEvent("PlayerTemplate_RewardIDUpdate", self, id)
    BNWoWPlayerHelper:FireCallbackEvent("PlayerTemplate_RewardIDUpdate", self, id)
end

function BNWoWPlayer:GetGGRewardID()
    if self.isPlayer then
        return GGL.db.Settings.Main.rewardID or 0
    else
        return GGL.db.PlayerRegister[self.syncID].rewardID or 0
    end
end

function BNWoWPlayer:SetGGLData(version)
    if version then
        self.gglVersion = version
    end
end

function BNWoWPlayer:GetDBTable()
    return GGL.db.PlayerRegister[self.syncID]
end

function BNWoWPlayer:UpdateMergeID()
    local newMergeID
    if self.isPlayerBattleTag then
        newMergeID = "self"
    elseif GGL.db.PlayerRegister[self.syncID].accountID then
        newMergeID = format(MERGE_ID_FORMAT, self.bnPlayer.battleTag, GGL.db.PlayerRegister[self.syncID].accountID)
    else
        newMergeID = self.syncID
    end
    if self.mergeID ~= newMergeID then
        local oldMergeID = self.mergeID
        if GGL.db.MergeIDNames[oldMergeID] then
            GGL.db.MergeIDNames[newMergeID] = GGL.db.PlayerRegister[oldMergeID]
        end
        self.mergeID = newMergeID
        self:FireCallbackEvent("PlayerTemplate_MergeIDUpdate", self, newMergeID, oldMergeID)
        BNWoWPlayerHelper:FireCallbackEvent("PlayerTemplate_MergeIDUpdate", self, newMergeID, oldMergeID)
    end
end

function BNWoWPlayer:IsSameWowProjekt()
    return self.isSameWowProjekt
end

function BNWoWPlayer:CanSendInviteToGameVersion()
    return self.wowProjectID and GGL.ALLOWED_PROJECT_ID_INV_SEND[self.wowProjectID] or false
end

--- NEVER call this directly!!!
function BNWoWPlayer:SetAccountID(accountID)
    local oldAccountID = GGL.db.PlayerRegister[self.syncID].accountID
    if oldAccountID == accountID then return false end
    GGL.db.PlayerRegister[self.syncID].accountID = accountID
    self:UpdateMergeID()
    self:FireCallbackEvent("PlayerTemplate_AccountIDUpdate", self, accountID, oldAccountID)
    return true
end

function BNWoWPlayer:GetGameVersionDisplayString()
    if self.wowProjectID and not self.isSameWowProjekt and GGL.PROJECT_ID_DISPLAY_STRINGS[self.wowProjectID] then
        return GGL.PROJECT_ID_DISPLAY_STRINGS[self.wowProjectID]
    end
    return ""
end

function BNWoWPlayer:GetAccountID()
    return GGL.db.PlayerRegister[self.syncID].accountID
end

function BNWoWPlayer:CanSendMessage(ignoreGGL)
    if self.isDummy then
        return false, "isDummy"
    elseif self.isPlayer then
        return false, "isPlayer"
    elseif ignoreGGL then
        return true, "ignoreGGL"
    elseif not self.gglVersion then
        return false, "gglVersion"
    else
        return true, "nil"
    end
end

function BNWoWPlayer:CompareWithPlayer(player)
    if not player then return end
    if player.isPlayer and self.isPlayer then
        return true, true, true
    end
    local samePlayer, sameBnetAccount, sameWoWAccount
    if self.syncID == player.syncID then
        samePlayer = true
    end
    if self.battleTag == player.battleTag then
        sameBnetAccount = true
    end
    if self:GetAccountID() == player:GetAccountID() then
        sameWoWAccount = true
    end

    return samePlayer, sameBnetAccount, sameWoWAccount
end

-- used for sender just before the send event only
function BNWoWPlayer:CanSendMessageNow()
    if not self.isOnline then
        return false
    elseif self.isPlayer then
        return false
    else
        return true
    end
end

function BNWoWPlayer:CanSendMessageNowRejectMessage()
    if not self.isOnline then
        return "isOnline"
    elseif self.isPlayer then
        return "isPlayer"
    else
        return "nil"
    end
end

function BNWoWPlayer:SetTimeOffset(offset)
    if offset == 0 then offset = nil end
    GGL.db.PlayerRegister[self.syncID].timeOffset = offset
end

function BNWoWPlayer:GetTimeOffset()
    return GGL.db.PlayerRegister[self.syncID].timeOffset or 0
end

function BNWoWPlayer:SetOnline(state)
    if self.isOnline ~= state then
        self:Update()
        --[[
        if self.isOnline ~= state then
            self.isOnline = state
            self:FireCallbackEvent("PlayerTemplate_PlayerUpdate", self)
            BNWoWPlayerHelper:FireCallbackEvent("PlayerTemplate_PlayerUpdate", self)
        end
        ]]
    end
end

function BNWoWPlayer:SetLastSeen(lastSeenTime)
    if not self.bnPlayer then return end
    self.bnPlayer:SetLastSeen(lastSeenTime)
end

function BNWoWPlayer:GetLastSeen()
    if not self.bnPlayer then return end
    return self.bnPlayer:GetLastSeen()
end